### Name: tk2dde
### Title: Use DDE (Dynamic Data Exchange) under Windows
### Aliases: tk2dde tk2dde.exec tk2dde.poke tk2dde.request tk2dde.services
### Keywords: utilities IO

### ** Examples

## Not run: 
##D ## These cannot be run by examples() but should be OK when pasted
##D ## into an interactive R session with the tcltk package loaded
##D 
##D ## Examples of DDE - Windows only
##D 
##D #### Examples using wish ####
##D # Start a Wish84 console side-by-side with R.
##D # (to get wish, you need to install ActiveTcl from
##D # http://www.activestate.com/Products/ActiveTcl/)
##D # Once it is done, start 'Wish84' from the start menu)
##D # Register the Wish console as a DDE server, that is, type in it
##D # (% is the Tcl prompt, don't type it!):
##D # % package require dde
##D # % dde servername wish
##D 
##D # In R:
##D tk2dde("R") # Return 0 if succeed
##D tk2dde.services()
##D # Evaluate some string in wish
##D tk2dde.exec("TclEval", "wish", "{puts {Hello World!}}")
##D # Give a value to a variable in wish
##D tk2dde.poke("TclEval", "wish", "myvar", "{This is a string!}")
##D # Note that you must surround strings with curly braces in Tcl!
##D tk2dde.poke("TclEval", "wish", "mynumvar", c(34.56, 78.9))
##D 
##D # In wish, check that vars exist and have correct value
##D # % puts $myvar
##D # % puts $mynumvar
##D 
##D # Get the value of one variable from wish into R
##D tk2dde.request("TclEval", "wish", "myvar")
##D tk2dde.request("TclEval", "wish", "mynumvar")
##D # Note that you don't know here if it is a string, a number, or so...
##D # You have to know and convert yourself!
##D 
##D # Now, the other way: execute a R function from wish
##D # You first need to register a R function for callback
##D # (For the moment, only functions without arguments are supported!)
##D doDDE <- function() cat("DDE execute!\n")       # A simple function
##D tclFun(doDDE)
##D # And in wish
##D # % dde execute TclEval R doDDE
##D 
##D # Once you have defined a variable using tclVar, you can get or change it
##D # from the dde server. However, tclVar gives cryptic names like ::RTcl1.
##D # So we prefer to use tclVarName
##D myvar2 <- tclVarName("myvar2", "this is a test...")
##D tclvalue(myvar2) # This is the way we access to this variable in R
##D 
##D # In wish you get the value and change it:
##D # % dde request TclEval R myvar2
##D # Again, dde poke does not work and must be replaced by an execute command
##D # This doesn't work (???)
##D # % dde poke TclEval R myvar2 {yes! and it works...}
##D # ... but this is fine
##D # % dde execute TclEval R {set myvar2 {yes! and it works...}}
##D 
##D # And in R...
##D tclvalue(myvar2)
##D 
##D #### DDE at the command line with execdde.exe ####
##D # You can also change the value of a variable, or run a command in R from
##D # the command line using execdde.exe:
##D # - Copy %R_Home%\library\tcltk2\bin\execdde.exe somewhere in your path,
##D # - Start a DOS window
##D # - Enter the following commands ('>' is the prompt, don't type it):
##D # > execdde -s TclEval -t R -c doDDE > NUL
##D # > if errorlevel 1 echo An error occurs... branch accordingly in your batch!
##D # > execdde -s TclEval -t R -c "set myvar2 {ok from execdde!}" > NUL
##D 
##D # And in R:
##D tclvalue(myvar2)
##D # Note: thanks to separate event loops, it works also when R calculates...
##D 
##D # Other examples of DDE use:
##D 
##D #### Manipulating Microsoft Excel ####
##D # Start Excel with a blank workbook, then...
##D 
##D # Change values in Excel from R:
##D tk2dde.poke("Excel", "Sheet1", "R1C1:R2C1", c("5.7", "6.34"))   # Some data
##D tk2dde.poke("Excel", "Sheet1", "R3C1", "= A1 + A2")                     # A formula
##D 
##D # Read values in Excel (note that results of formulas are returned)
##D Res <- tk2dde.request("Excel", "Sheet1", "R1C1:R3C1")
##D Res
##D as.numeric(Res)
##D 
##D # Execute a command in Excel
##D tk2dde.exec("Excel", "Sheet1", "{\[\Select(\"R1C1:R3C1\")]\[New(2,2)\]}")
##D # New(2,2) create a bar graph in a separate sheet
##D # Note: close this graph now, in order to run the rest properly!
##D 
##D # R as server and Excel as client
##D # Prepare R as server
##D tk2dde("R") # Now, my server name is TclEval|R !
##D # Create a variable that will be shared with Excel
##D XlVar <- tclVarName("XlVar", "this is a test!")
##D # Enter a formula somewhere in Excel to link to this variable
##D tk2dde.poke("Excel", "Sheet1", "R5C1", "=TclEval|R!'::XlVar'")
##D 
##D # Now, change the content in XlVar
##D tclvalue(XlVar) <- "Another text..."
##D # Rem: in my Excel, the content of the cell is not changed automatically...
##D # I must reenter the formula for the changes to be considered!?
##D 
##D #### Manipulating Microsoft Word ####
##D # Start Word and then...
##D tk2dde.exec("Winword", "System", "{\[EndOfDocument\]}")
##D tk2dde.exec("Winword", "System", "{\[Insert \"Text from R.\"\]}",
##D         async = TRUE)
##D 
##D # Can also use:
##D #  AppMaximize 1
##D #  StatOfDocument
##D #  EditFind .Find="hello" .PatternMatch=1
##D #  FileExit 2
##D #  FileClose 2 # 2 = close without saving, 1 = save first, 0 = prompt
##D 
##D # If you want to start the program if not running, you must take care that
##D # you leave enough time to the program to initialize before you can use DDE
##D # commands with it! Here is an example (make sure that Word is not running):
##D is.WordStarted <- function()
##D         (length(tk2dde.services("Winword", "System")) > 0)
##D if (!is.WordStarted()) {
##D         # Get the path to WinWord.exe
##D         Word <- try(tk2reg.get(paste("HKEY_LOCAL_MACHINE", "SOFTWARE", "Classes",
##D                 "WordDocument", "protocol", "StdFileEditing", "server", sep = "\"), ""))
##D         if (is.na(Word)) {
##D                 cat("Microsoft Word is not found!\n")
##D         } else {        # Start Microsoft Word and wait for its complete loading
##D                 # Start Word
##D                 system(Word, wait = FALSE)
##D                 # Wait a maximum of 15 sec...
##D                 for (i in 1:15) {
##D                         Sys.sleep(1)    # Wait 1 sec
##D                         res <- is.WordStarted()
##D                         if (res) break
##D                 }
##D                 if (res) cat("Now Word is started and ready to get DDE commands!\n")
##D                 tk2dde.exec("Winword", "System", "{\[Insert \"My text from R.\"\]}",
##D                         async = TRUE)
##D         }
##D }
##D 
##D #### Control Matlab
##D # Start Matlab and then...
##D tk2dde.exec("Matlab", "Engine", "s = ones(5);")
##D # Then in Matlab, type s
##D 
##D #### Manipulating Progman to create and delete shortcuts ####
##D tk2dde.exec("progman", "progman", "{\[CreateGroup(Bogus)\]}")
##D tk2dde.exec("progman", "progman", "{\[AddItem(notepad.exe,BogusPadLink)\]}")
##D tk2dde.exec("progman", "progman", "{\[ShowGroup(Bogus,0)\]}")
##D tk2dde.exec("progman", "progman", "{\[ShowGroup(Bogus,1)\]}")
##D # And delete it...
##D tk2dde.exec("progman", "progman", "{\[DeleteItem(BogusPadLink)\]}")
##D tk2dde.exec("progman", "progman", "{\[DeleteGroup(Bogus)\]}")
##D ## Rem: the same can be done with tcom, if installed:
##D #.Tcl("package require tcom")
##D #.Tcl("set sh [::tcom::ref createobject {WScript.Shell}]")
##D #.Tcl("set lnk [$sh CreateShortcut {c:\\temp\\boguspad.lnk}]")
##D #.Tcl("$lnk TargetPath {\"notepad.exe\"}")
##D #.Tcl("$lnk WorkingDirectory {c:\\temp}")
##D #.Tcl("$lnk Arguments Tutorial.txt")
##D #.Tcl("$lnk Save")
##D 
##D #### Controlling Internet Explorer ####
##D # Start Internet Explorer, then...
##D tk2dde.exec("iexplore", "WWW_OpenURL", "{http://www.r-project.org/}")
##D # It works, but return an error in R???
##D # This is similar to the browseURL() in package util (just for the demo!)
##D HelpIndex <- file.path(R.home(), "doc", "html", "index.html")
##D tk2dde.exec("iexplore", "WWW_ShowFile", paste("{", HelpIndex, "}", sep = ""))
##D # Info on the current opened window
##D tk2dde.request("iexplore", "WWW_GetWindowInfo", "1")
##D 
##D #### Controlling Windows explorer ####
##D tk2dde.exec("Folders", "AppProperties",
##D         "{\[ViewFolder(\"C:\\",\"C:\\",5)\]}")
##D # ... or you can also try 'ExplodeFolder'
##D     
##D # Search in folder
##D tk2dde.exec("Folders", "AppProperties", "{\[FindFolder(\"\",\"C:\\")\]}")
## End(Not run)



